@@ -115,6 +115,8 @@ import org.apache.tomcat.util.net.IPv6Utils;
  * <li><code>%{xxx}s</code> xxx is an attribute in the HttpSession
  * <li><code>%{xxx}t</code> xxx is an enhanced SimpleDateFormat pattern (see Configuration Reference document for
  * details on supported time patterns)
+ * <li><code>%{xxx}L</code> xxx is the identifier to log (see Configuration Reference document for details on supported
+ * identifiers)
  * <li><code>%{xxx}T</code> xxx is the unit for the time taken to process the request (see Configuration Reference
  * document for details on supported units)
  * </ul>
@@ -167,7 +169,12 @@ public abstract class AbstractAccessLogValve extends ValveBase implements Access
         PEER
     }
 
-    // ------------------------------------------------------ Constructor
+    private enum IdentifierType {
+        CONNECTION,
+        UNKNOWN
+    }
+
+
     public AbstractAccessLogValve() {
         super(true);
     }
@@ -362,7 +369,7 @@ public abstract class AbstractAccessLogValve extends ValveBase implements Access
         private final Locale cacheDefaultLocale;
         private final DateFormatCache parent;
         protected final Cache cLFCache;
-        private final Map<String, Cache> formatCache = new HashMap<>();
+        private final Map<String,Cache> formatCache = new HashMap<>();
 
         protected DateFormatCache(int size, Locale loc, DateFormatCache parent) {
             cacheSize = size;
@@ -409,14 +416,14 @@ public abstract class AbstractAccessLogValve extends ValveBase implements Access
     /**
      * Global date format cache.
      */
-    private static final DateFormatCache globalDateCache = new DateFormatCache(globalCacheSize, Locale.getDefault(),
-            null);
+    private static final DateFormatCache globalDateCache =
+            new DateFormatCache(globalCacheSize, Locale.getDefault(), null);
 
     /**
      * Thread local date format cache.
      */
-    private static final ThreadLocal<DateFormatCache> localDateCache = ThreadLocal
-            .withInitial(() -> new DateFormatCache(localCacheSize, Locale.getDefault(), globalDateCache));
+    private static final ThreadLocal<DateFormatCache> localDateCache =
+            ThreadLocal.withInitial(() -> new DateFormatCache(localCacheSize, Locale.getDefault(), globalDateCache));
 
 
     /**
@@ -1341,7 +1348,7 @@ public abstract class AbstractAccessLogValve extends ValveBase implements Access
             /**
              * Append the time to the buffer in the appropriate format.
              *
-             * @param buf The buffer to append to.
+             * @param buf  The buffer to append to.
              * @param time The time to log in nanoseconds.
              */
             public abstract void append(CharArrayWriter buf, long time);
@@ -1677,6 +1684,48 @@ public abstract class AbstractAccessLogValve extends ValveBase implements Access
         }
     }
 
+
+
+    /**
+     * Write identifier element %{xxx}L
+     */
+    protected class IdentifierElement implements AccessLogElement {
+
+        /**
+         * Type of identifier to log
+         */
+        private final IdentifierType identifierType;
+
+        public IdentifierElement() {
+            this(null);
+        }
+
+
+        public IdentifierElement(String type) {
+            switch (type) {
+                case "c":
+                    identifierType = IdentifierType.CONNECTION;
+                    break;
+                default:
+                    log.error(sm.getString("accessLogValve.invalidIdentifierType", type));
+                    identifierType = IdentifierType.UNKNOWN;
+                    break;
+            }
+        }
+
+        @Override
+        public void addElement(CharArrayWriter buf, Date date, Request request, Response response, long time) {
+            switch(identifierType) {
+                case CONNECTION:
+                    buf.append(request.getServletConnection().getConnectionId());
+                    break;
+                case UNKNOWN:
+                    buf.append("???");
+            }
+        }
+    }
+
+
     /**
      * Parse pattern string and create the array of AccessLogElement.
      *
@@ -1749,14 +1798,16 @@ public abstract class AbstractAccessLogValve extends ValveBase implements Access
      */
     protected AccessLogElement createAccessLogElement(String name, char pattern) {
         switch (pattern) {
-            case 'i':
-                return new HeaderElement(name);
-            case 'c':
-                return new CookieElement(name);
-            case 'o':
-                return new ResponseHeaderElement(name);
             case 'a':
                 return new RemoteAddrElement(name);
+            case 'c':
+                return new CookieElement(name);
+            case 'i':
+                return new HeaderElement(name);
+            case 'L':
+                return new IdentifierElement(name);
+            case 'o':
+                return new ResponseHeaderElement(name);
             case 'p':
                 return new PortElement(name);
             case 'r':
@@ -1917,7 +1968,7 @@ public abstract class AbstractAccessLogValve extends ValveBase implements Access
                     // is encountered plus at the end.
                     default:
                 }
-            // Control (1-31), delete (127) or above 127
+                // Control (1-31), delete (127) or above 127
             } else {
                 // Write unchanged string parts
                 if (current > next) {
