@@ -20,10 +20,10 @@ import java.io.CharArrayWriter;
 import java.io.FileNotFoundException;
 import java.util.Collection;
 
-import javax.servlet.jsp.tagext.TagAttributeInfo;
-import javax.servlet.jsp.tagext.TagFileInfo;
-import javax.servlet.jsp.tagext.TagInfo;
-import javax.servlet.jsp.tagext.TagLibraryInfo;
+import jakarta.servlet.jsp.tagext.TagAttributeInfo;
+import jakarta.servlet.jsp.tagext.TagFileInfo;
+import jakarta.servlet.jsp.tagext.TagInfo;
+import jakarta.servlet.jsp.tagext.TagLibraryInfo;
 
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
@@ -36,7 +36,12 @@ import org.xml.sax.helpers.AttributesImpl;
 /**
  * This class implements a parser for a JSP page (non-xml view). JSP page grammar is included here for reference. The
  * token '#' that appears in the production indicates the current input token location in the production.
+ *
+ * @author Kin-man Chung
+ * @author Shawn Bayern
+ * @author Mark Roth
  */
+
 class Parser implements TagConstants {
 
     private final ParserController parserController;
@@ -67,12 +72,6 @@ class Parser implements TagConstants {
 
     private static final String JAVAX_BODY_CONTENT_TEMPLATE_TEXT = "JAVAX_BODY_CONTENT_TEMPLATE_TEXT";
 
-    /*
-     * System property that controls if the strict white space rules are applied.
-     */
-    private static final boolean STRICT_WHITESPACE =
-            Boolean.parseBoolean(System.getProperty("org.apache.jasper.compiler.Parser.STRICT_WHITESPACE", "true"));
-
     /**
      * The constructor
      */
@@ -114,7 +113,8 @@ class Parser implements TagConstants {
 
         Parser parser = new Parser(pc, reader, isTagFile, directivesOnly, jar);
 
-        Node.Root root = new Node.Root(reader.mark(), parent, false);
+        Node.Root root = new Node.Root(reader.mark(), parent, false,
+                pc.getJspCompilationContext().getOptions().getTempVariableNamePrefix());
         root.setPageEncoding(pageEnc);
         root.setJspConfigPageEncoding(jspConfigPageEnc);
         root.setIsDefaultPageEncoding(isDefaultPageEncoding);
@@ -136,7 +136,8 @@ class Parser implements TagConstants {
             parser.addInclude(root, pageInfo.getIncludeCoda());
         }
 
-        return new Node.Nodes(root);
+        Node.Nodes page = new Node.Nodes(root);
+        return page;
     }
 
     /**
@@ -154,7 +155,7 @@ class Parser implements TagConstants {
 
         try {
             while (parseAttribute(attrs)) {
-                if (ws == 0 && STRICT_WHITESPACE) {
+                if (ws == 0 && ctxt.getOptions().getStrictWhitespace()) {
                     err.jspError(reader.mark(), "jsp.error.attribute.nowhitespace");
                 }
                 ws = reader.skipSpaces();
@@ -243,12 +244,15 @@ class Parser implements TagConstants {
         char ch = (char) reader.peekChar();
         if (Character.isLetter(ch) || ch == '_' || ch == ':') {
             StringBuilder buf = new StringBuilder();
-            do {
+            buf.append(ch);
+            reader.nextChar();
+            ch = (char) reader.peekChar();
+            while (Character.isLetter(ch) || Character.isDigit(ch) || ch == '.' || ch == '_' || ch == '-' ||
+                    ch == ':') {
                 buf.append(ch);
                 reader.nextChar();
                 ch = (char) reader.peekChar();
-            } while (Character.isLetter(ch) || Character.isDigit(ch) || ch == '.' || ch == '_' || ch == '-' ||
-                    ch == ':');
+            }
             return buf.toString();
         }
         return null;
@@ -330,8 +334,8 @@ class Parser implements TagConstants {
             }
         } catch (FileNotFoundException ex) {
             err.jspError(start, "jsp.error.file.not.found", file);
-        } catch (Exception e) {
-            err.jspError(start, e.getMessage());
+        } catch (Exception ex) {
+            err.jspError(start, ex.getMessage());
         }
     }
 
@@ -564,7 +568,7 @@ class Parser implements TagConstants {
     }
 
     /*
-     * Parses an attribute directive with the following syntax: AttributeDirective ::= ( S Attribute)*
+     * Parses a attribute directive with the following syntax: AttributeDirective ::= ( S Attribute)*
      */
     private void parseAttributeDirective(Node parent) throws JasperException {
         Attributes attrs = parseAttributes();
@@ -907,8 +911,11 @@ class Parser implements TagConstants {
     }
 
     /*
-     * Parses OptionalBody, but also reused to parse bodies for plugin and param since the syntax is identical (the only
-     * thing that differs substantially is how to process the body, and thus we accept the body type as a parameter).
+     * <<<<<<< HEAD Parses OptionalBody, but also reused to parse bodies for plugin and param since the syntax is
+     * identical (the only thing that differs substantially is how to process the body, and thus we accept the body type
+     * as a parameter). ======= Parses OptionalBody, but also reused to parse bodies for param since the syntax is
+     * identical (the only thing that differs substantially is how to process the body, and thus we accept the body type
+     * as a parameter). >>>>>>> 76d2ad0a5d (Code clean-up. Formatting. No functional change.)
      *
      * OptionalBody ::= EmptyBody | ActionBody
      *
@@ -957,7 +964,8 @@ class Parser implements TagConstants {
         boolean result = false;
 
         if (reader.matchesOptionalSpacesFollowedBy("<jsp:attribute")) {
-            // It may be an EmptyBody, depending on whether there's a "<jsp:body" before the ETag
+            // May be an EmptyBody, depending on whether
+            // There's a "<jsp:body" before the ETag
 
             // First, parse <jsp:attribute> elements:
             parseNamedAttributes(parent);
@@ -984,8 +992,8 @@ class Parser implements TagConstants {
     }
 
     /*
-     * Params ::= `>' S? ( ( `<jsp:body>' ( ( S? Param+ S? `</jsp:body>' ) | <TRANSLATION_ERROR> ) ) | Param+ )
-     * '</jsp:params>'
+     * <<<<<<< HEAD Params ::= `>' S? ( ( `<jsp:body>' ( ( S? Param+ S? `</jsp:body>' ) | <TRANSLATION_ERROR> ) ) |
+     * Param+ ) '</jsp:params>'
      */
     private void parseJspParams(Node parent) throws JasperException {
         Node jspParamsNode = new Node.ParamsAction(start, parent);
@@ -1184,7 +1192,7 @@ class Parser implements TagConstants {
         // 'CustomActionJSPContent', or 'CustomActionScriptlessContent'.
         // depending on body-content in TLD.
 
-        // Looking for a body, it still can be empty; but if there is
+        // Looking for a body, it still can be empty; but if there is a
         // a tag body, its syntax would be dependent on the type of
         // body content declared in the TLD.
         String bc;
@@ -1194,7 +1202,7 @@ class Parser implements TagConstants {
             bc = tagFileInfo.getTagInfo().getBodyContent();
         }
 
-        Node tagNode;
+        Node tagNode = null;
         if (tagInfo != null) {
             tagNode = new Node.CustomTag(tagName, prefix, shortTagName, uri, attrs, start, parent, tagInfo,
                     tagHandlerClass);
