@@ -23,15 +23,15 @@ import java.util.List;
 import java.util.Map;
 import java.util.Map.Entry;
 
-import javax.el.MethodExpression;
-import javax.el.ValueExpression;
-import javax.servlet.jsp.tagext.JspFragment;
-import javax.servlet.jsp.tagext.TagAttributeInfo;
-import javax.servlet.jsp.tagext.TagFileInfo;
-import javax.servlet.jsp.tagext.TagInfo;
-import javax.servlet.jsp.tagext.TagLibraryInfo;
-import javax.servlet.jsp.tagext.TagVariableInfo;
-import javax.servlet.jsp.tagext.VariableInfo;
+import jakarta.el.MethodExpression;
+import jakarta.el.ValueExpression;
+import jakarta.servlet.jsp.tagext.JspFragment;
+import jakarta.servlet.jsp.tagext.TagAttributeInfo;
+import jakarta.servlet.jsp.tagext.TagFileInfo;
+import jakarta.servlet.jsp.tagext.TagInfo;
+import jakarta.servlet.jsp.tagext.TagLibraryInfo;
+import jakarta.servlet.jsp.tagext.TagVariableInfo;
+import jakarta.servlet.jsp.tagext.VariableInfo;
 
 import org.apache.jasper.JasperException;
 import org.apache.jasper.JspCompilationContext;
@@ -46,7 +46,7 @@ import org.apache.tomcat.util.descriptor.tld.TldResourcePath;
  * @author Kin-man Chung
  */
 
-public class TagFileProcessor {
+class TagFileProcessor {
 
     private List<Compiler> tempVector;
 
@@ -61,7 +61,8 @@ public class TagFileProcessor {
                 new JspUtil.ValidAttribute("description"), new JspUtil.ValidAttribute("example"),
                 new JspUtil.ValidAttribute("pageEncoding"), new JspUtil.ValidAttribute("language"),
                 new JspUtil.ValidAttribute("import"), new JspUtil.ValidAttribute("deferredSyntaxAllowedAsLiteral"),
-                new JspUtil.ValidAttribute("trimDirectiveWhitespaces"), new JspUtil.ValidAttribute("isELIgnored") };
+                new JspUtil.ValidAttribute("trimDirectiveWhitespaces"), new JspUtil.ValidAttribute("isELIgnored"),
+                new JspUtil.ValidAttribute("errorOnELNotFound") };
 
         private static final JspUtil.ValidAttribute[] attributeDirectiveAttrs = {
                 new JspUtil.ValidAttribute("name", true), new JspUtil.ValidAttribute("required"),
@@ -76,15 +77,15 @@ public class TagFileProcessor {
                         new JspUtil.ValidAttribute("scope"), new JspUtil.ValidAttribute("declare"),
                         new JspUtil.ValidAttribute("description") };
 
-        private final ErrorDispatcher err;
+        private ErrorDispatcher err;
 
-        private final TagLibraryInfo tagLibInfo;
+        private TagLibraryInfo tagLibInfo;
 
-        private final String name;
+        private String name = null;
 
-        private final String path;
+        private String path = null;
 
-        private String bodyContent = null;
+        private String bodycontent = null;
 
         private String description = null;
 
@@ -98,9 +99,9 @@ public class TagFileProcessor {
 
         private String example = null;
 
-        private final List<TagAttributeInfo> attributeList;
+        private List<TagAttributeInfo> attributeList;
 
-        private final List<TagVariableInfo> variableList;
+        private List<TagVariableInfo> variableList;
 
         private static final String ATTR_NAME = "the name attribute of the attribute directive";
 
@@ -112,9 +113,9 @@ public class TagFileProcessor {
 
         private static final String TAG_DYNAMIC = "the dynamic-attributes attribute of the tag directive";
 
-        private final Map<String,NameEntry> nameTable = new HashMap<>();
+        private Map<String,NameEntry> nameTable = new HashMap<>();
 
-        private final Map<String,NameEntry> nameFromTable = new HashMap<>();
+        private Map<String,NameEntry> nameFromTable = new HashMap<>();
 
         TagFileDirectiveVisitor(Compiler compiler, TagLibraryInfo tagLibInfo, String name, String path) {
             err = compiler.getErrorDispatcher();
@@ -130,11 +131,11 @@ public class TagFileProcessor {
 
             JspUtil.checkAttributes("Tag directive", n, tagDirectiveAttrs, err);
 
-            bodyContent = checkConflict(n, bodyContent, "body-content");
-            if (bodyContent != null && !bodyContent.equalsIgnoreCase(TagInfo.BODY_CONTENT_EMPTY) &&
-                    !bodyContent.equalsIgnoreCase(TagInfo.BODY_CONTENT_TAG_DEPENDENT) &&
-                    !bodyContent.equalsIgnoreCase(TagInfo.BODY_CONTENT_SCRIPTLESS)) {
-                err.jspError(n, "jsp.error.tagdirective.badbodycontent", bodyContent);
+            bodycontent = checkConflict(n, bodycontent, "body-content");
+            if (bodycontent != null && !bodycontent.equalsIgnoreCase(TagInfo.BODY_CONTENT_EMPTY) &&
+                    !bodycontent.equalsIgnoreCase(TagInfo.BODY_CONTENT_TAG_DEPENDENT) &&
+                    !bodycontent.equalsIgnoreCase(TagInfo.BODY_CONTENT_SCRIPTLESS)) {
+                err.jspError(n, "jsp.error.tagdirective.badbodycontent", bodycontent);
             }
             dynamicAttrsMapName = checkConflict(n, dynamicAttrsMapName, "dynamic-attributes");
             if (dynamicAttrsMapName != null) {
@@ -315,31 +316,31 @@ public class TagFileProcessor {
             variableList.add(new TagVariableInfo(nameGiven, nameFromAttribute, className, declare, scope));
         }
 
-        public TagInfo getTagInfo() throws JasperException {
+        public TagInfo getTagInfo(String packageName) throws JasperException {
 
             if (name == null) {
                 // XXX Get it from tag file name
             }
 
-            if (bodyContent == null) {
-                bodyContent = TagInfo.BODY_CONTENT_SCRIPTLESS;
+            if (bodycontent == null) {
+                bodycontent = TagInfo.BODY_CONTENT_SCRIPTLESS;
             }
 
-            String tagClassName = JspUtil.getTagHandlerClassName(path, tagLibInfo.getReliableURN(), err);
+            String tagClassName = JspUtil.getTagHandlerClassName(path, packageName, tagLibInfo.getReliableURN(), err);
 
             TagVariableInfo[] tagVariableInfos = variableList.toArray(new TagVariableInfo[0]);
             TagAttributeInfo[] tagAttributeInfo = attributeList.toArray(new TagAttributeInfo[0]);
 
-            return new JasperTagInfo(name, tagClassName, bodyContent, description, tagLibInfo, null, tagAttributeInfo,
+            return new JasperTagInfo(name, tagClassName, bodycontent, description, tagLibInfo, null, tagAttributeInfo,
                     displayName, smallIcon, largeIcon, tagVariableInfos, dynamicAttrsMapName);
         }
 
         static class NameEntry {
-            private final String type;
+            private String type;
 
-            private final Node node;
+            private Node node;
 
-            private final TagAttributeInfo attr;
+            private TagAttributeInfo attr;
 
             NameEntry(String type, Node node, TagAttributeInfo attr) {
                 this.type = type;
@@ -440,7 +441,7 @@ public class TagFileProcessor {
         Node.Nodes page = null;
         try {
             page = pc.parseTagFileDirectives(path, jar);
-        } catch (IOException ioe) {
+        } catch (IOException e) {
             err.jspError("jsp.error.file.not.found", path);
         }
 
@@ -448,7 +449,7 @@ public class TagFileProcessor {
         page.visit(tagFileVisitor);
         tagFileVisitor.postCheck();
 
-        return tagFileVisitor.getTagInfo();
+        return tagFileVisitor.getTagInfo(pc.getJspCompilationContext().getOptions().getGeneratedTagFilePackageName());
     }
 
     /**
@@ -559,9 +560,9 @@ public class TagFileProcessor {
      */
     private class TagFileLoaderVisitor extends Node.Visitor {
 
-        private final Compiler compiler;
+        private Compiler compiler;
 
-        private final PageInfo pageInfo;
+        private PageInfo pageInfo;
 
         TagFileLoaderVisitor(Compiler compiler) {
 
@@ -624,7 +625,7 @@ public class TagFileProcessor {
     /**
      * Removed the java and class files for the tag prototype generated from the current compilation.
      *
-     * @param classFileName If non-null, remove only the class file with this name.
+     * @param classFileName If non-null, remove only the class file with with this name.
      */
     public void removeProtoTypeFiles(String classFileName) {
         for (Compiler c : tempVector) {
