@@ -27,18 +27,18 @@ import java.util.Map;
 import java.util.regex.Matcher;
 import java.util.regex.Pattern;
 
-import javax.el.ELException;
-import javax.el.ExpressionFactory;
-import javax.el.FunctionMapper;
-import javax.servlet.jsp.JspFactory;
-import javax.servlet.jsp.tagext.FunctionInfo;
-import javax.servlet.jsp.tagext.PageData;
-import javax.servlet.jsp.tagext.TagAttributeInfo;
-import javax.servlet.jsp.tagext.TagData;
-import javax.servlet.jsp.tagext.TagExtraInfo;
-import javax.servlet.jsp.tagext.TagInfo;
-import javax.servlet.jsp.tagext.TagLibraryInfo;
-import javax.servlet.jsp.tagext.ValidationMessage;
+import jakarta.el.ELException;
+import jakarta.el.ExpressionFactory;
+import jakarta.el.FunctionMapper;
+import jakarta.servlet.jsp.JspFactory;
+import jakarta.servlet.jsp.tagext.FunctionInfo;
+import jakarta.servlet.jsp.tagext.PageData;
+import jakarta.servlet.jsp.tagext.TagAttributeInfo;
+import jakarta.servlet.jsp.tagext.TagData;
+import jakarta.servlet.jsp.tagext.TagExtraInfo;
+import jakarta.servlet.jsp.tagext.TagInfo;
+import jakarta.servlet.jsp.tagext.TagLibraryInfo;
+import jakarta.servlet.jsp.tagext.ValidationMessage;
 
 import org.apache.jasper.JasperException;
 import org.apache.jasper.compiler.ELNode.Text;
@@ -66,15 +66,16 @@ class Validator {
 
         private final ErrorDispatcher err;
 
-        private static final JspUtil.ValidAttribute[] pageDirectiveAttrs = { new JspUtil.ValidAttribute("language"),
-                new JspUtil.ValidAttribute("extends"), new JspUtil.ValidAttribute("import"),
-                new JspUtil.ValidAttribute("session"), new JspUtil.ValidAttribute("buffer"),
-                new JspUtil.ValidAttribute("autoFlush"), new JspUtil.ValidAttribute("isThreadSafe"),
-                new JspUtil.ValidAttribute("info"), new JspUtil.ValidAttribute("errorPage"),
-                new JspUtil.ValidAttribute("isErrorPage"), new JspUtil.ValidAttribute("contentType"),
-                new JspUtil.ValidAttribute("pageEncoding"), new JspUtil.ValidAttribute("isELIgnored"),
-                new JspUtil.ValidAttribute("deferredSyntaxAllowedAsLiteral"),
-                new JspUtil.ValidAttribute("trimDirectiveWhitespaces") };
+        private static final JspUtil.ValidAttribute[] pageDirectiveAttrs =
+                { new JspUtil.ValidAttribute("language"), new JspUtil.ValidAttribute("extends"),
+                        new JspUtil.ValidAttribute("import"), new JspUtil.ValidAttribute("session"),
+                        new JspUtil.ValidAttribute("buffer"), new JspUtil.ValidAttribute("autoFlush"),
+                        new JspUtil.ValidAttribute("isThreadSafe"), new JspUtil.ValidAttribute("info"),
+                        new JspUtil.ValidAttribute("errorPage"), new JspUtil.ValidAttribute("isErrorPage"),
+                        new JspUtil.ValidAttribute("contentType"), new JspUtil.ValidAttribute("pageEncoding"),
+                        new JspUtil.ValidAttribute("isELIgnored"), new JspUtil.ValidAttribute("errorOnELNotFound"),
+                        new JspUtil.ValidAttribute("deferredSyntaxAllowedAsLiteral"),
+                        new JspUtil.ValidAttribute("trimDirectiveWhitespaces") };
 
         private boolean pageEncodingSeen = false;
 
@@ -155,6 +156,13 @@ class Validator {
                     } else if (!pageInfo.getIsELIgnored().equals(value)) {
                         err.jspError(n, "jsp.error.page.conflict.iselignored", pageInfo.getIsELIgnored(), value);
                     }
+                } else if ("errorOnELNotFound".equals(attr)) {
+                    if (pageInfo.getErrorOnELNotFound() == null) {
+                        pageInfo.setErrorOnELNotFound(value, n, err, true);
+                    } else if (!pageInfo.getErrorOnELNotFound().equals(value)) {
+                        err.jspError(n, "jsp.error.page.conflict.errorOnELNotFound", pageInfo.getErrorOnELNotFound(),
+                                value);
+                    }
                 } else if ("isErrorPage".equals(attr)) {
                     if (pageInfo.getIsErrorPage() == null) {
                         pageInfo.setIsErrorPage(value, n, err);
@@ -233,6 +241,13 @@ class Validator {
                     } else if (!pageInfo.getIsELIgnored().equals(value)) {
                         err.jspError(n, "jsp.error.tag.conflict.iselignored", pageInfo.getIsELIgnored(), value);
                     }
+                } else if ("errorOnELNotFound".equals(attr)) {
+                    if (pageInfo.getErrorOnELNotFound() == null) {
+                        pageInfo.setErrorOnELNotFound(value, n, err, false);
+                    } else if (!pageInfo.getErrorOnELNotFound().equals(value)) {
+                        err.jspError(n, "jsp.error.tag.conflict.errorOnELNotFound", pageInfo.getErrorOnELNotFound(),
+                                value);
+                    }
                 } else if ("pageEncoding".equals(attr)) {
                     if (pageEncodingSeen) {
                         err.jspError(n, "jsp.error.tag.multi.pageencoding");
@@ -437,7 +452,7 @@ class Validator {
             JspUtil.checkAttributes("Jsp:root", n, jspRootAttrs, err);
             String version = n.getTextAttribute("version");
             if (!version.equals("1.2") && !version.equals("2.0") && !version.equals("2.1") && !version.equals("2.2") &&
-                    !version.equals("2.3")) {
+                    !version.equals("2.3") && !version.equals("3.0") && !version.equals("3.1")) {
                 err.jspError(n, "jsp.error.jsproot.version.invalid", version);
             }
             visitBody(n);
@@ -706,8 +721,10 @@ class Validator {
                 char c = value.charAt(i);
                 if (c == '#' && (i + 1) < len && value.charAt(i + 1) == '{' && !prevCharIsEscape) {
                     return true;
+                } else if (c == '\\') {
+                    prevCharIsEscape = true;
                 } else {
-                    prevCharIsEscape = c == '\\';
+                    prevCharIsEscape = false;
                 }
                 i++;
             }
@@ -1009,7 +1026,7 @@ class Validator {
                 }
                 for (int j = 0; tldAttrs != null && j < tldAttrs.length; j++) {
                     if (attrs.getLocalName(i).equals(tldAttrs[j].getName()) && (attrs.getURI(i) == null ||
-                            attrs.getURI(i).isEmpty() || attrs.getURI(i).equals(n.getURI()))) {
+                            attrs.getURI(i).length() == 0 || attrs.getURI(i).equals(n.getURI()))) {
 
                         TagAttributeInfo tldAttr = tldAttrs[j];
                         if (tldAttr.canBeRequestTime() || tldAttr.isDeferredMethod() || tldAttr.isDeferredValue()) { // JSP
@@ -1138,7 +1155,7 @@ class Validator {
                      */
                     String attrPrefix = na.getPrefix();
                     if (na.getLocalName().equals(tldAttr.getName()) &&
-                            (attrPrefix == null || attrPrefix.isEmpty() || attrPrefix.equals(n.getPrefix()))) {
+                            (attrPrefix == null || attrPrefix.length() == 0 || attrPrefix.equals(n.getPrefix()))) {
                         jspAttrs[start + i] = new Node.JspAttribute(na, tldAttr, false);
                         NamedAttributeVisitor nav = null;
                         if (na.getBody() != null) {
@@ -1167,10 +1184,10 @@ class Validator {
         /**
          * Preprocess attributes that can be expressions. Expression delimiters are stripped.
          * <p>
-         * If value is null, checks if there are any NamedAttribute sub elements in the tree node, and if so, constructs
+         * If value is null, checks if there are any NamedAttribute subelements in the tree node, and if so, constructs
          * a JspAttribute out of a child NamedAttribute node.
          *
-         * @param el EL expression, if already parsed by the caller (so that we can skip reparsing it)
+         * @param el EL expression, if already parsed by the caller (so that we can skip re-parsing it)
          */
         private Node.JspAttribute getJspAttribute(TagAttributeInfo tai, String qName, String uri, String localName,
                 String value, Node n, ELNode.Nodes el, boolean dynamic) throws JasperException {
@@ -1368,7 +1385,7 @@ class Validator {
 
             class FVVisitor extends ELNode.Visitor {
 
-                private final Node n;
+                private Node n;
 
                 FVVisitor(Node n) {
                     this.n = n;
@@ -1487,7 +1504,7 @@ class Validator {
 
             class ValidateFunctionMapper extends FunctionMapper {
 
-                private final Map<String,Method> fnmap = new HashMap<>();
+                private Map<String,Method> fnmap = new HashMap<>();
 
                 @Override
                 public void mapFunction(String prefix, String localName, Method method) {
@@ -1501,7 +1518,7 @@ class Validator {
             }
 
             class MapperELVisitor extends ELNode.Visitor {
-                private final ValidateFunctionMapper fmapper;
+                private ValidateFunctionMapper fmapper;
 
                 MapperELVisitor(ValidateFunctionMapper fmapper) {
                     this.fmapper = fmapper;
@@ -1524,9 +1541,9 @@ class Validator {
                         err.jspError("jsp.error.function.classnotfound", n.getFunctionInfo().getFunctionClass(),
                                 n.getPrefix() + ':' + n.getName(), e.getMessage());
                     }
-                    String[] paramTypes = n.getParameters();
+                    String paramTypes[] = n.getParameters();
                     int size = paramTypes.length;
-                    Class<?>[] params = new Class[size];
+                    Class<?> params[] = new Class[size];
                     int i = 0;
                     try {
                         for (i = 0; i < size; i++) {
